#
# This file was generated using xslt from its XML file
#
# Copyright 2014, Associated Universities Inc., Washington DC
#
import sys
import os
#from casac import *
import casac
import string
import time
import inspect
import gc
import numpy
from odict import odict
from types import * 
from task_sdcal2 import sdcal2
class sdcal2_cli_:
    __name__ = "sdcal2"
    rkey = None
    i_am_a_casapy_task = None
    # The existence of the i_am_a_casapy_task attribute allows help()
    # (and other) to treat casapy tasks as a special case.

    def __init__(self) :
       self.__bases__ = (sdcal2_cli_,)
       self.__doc__ = self.__call__.__doc__

       self.parameters={'infile':None, 'calmode':None, 'fraction':None, 'noff':None, 'width':None, 'elongated':None, 'tsysavg':None, 'tsysspw':None, 'applytable':None, 'interp':None, 'spwmap':None, 'field':None, 'spw':None, 'scan':None, 'pol':None, 'outfile':None, 'overwrite':None, }


    def result(self, key=None):
	    #### and add any that have completed...
	    return None


    def __call__(self, infile=None, calmode=None, fraction=None, noff=None, width=None, elongated=None, tsysavg=None, tsysspw=None, applytable=None, interp=None, spwmap=None, field=None, spw=None, scan=None, pol=None, outfile=None, overwrite=None, ):

        """ASAP SD calibration task

	Detailed Description: 

Task sdcal2 is an implementation of a calibration scheme like as 
interferometry, i.e., generate caltables and apply them. Available 
calibration modes are 'ps', 'otf', 'otfraster', and 'tsys'. Those 
modes generates caltables for sky or Tsys calibration. Those 
caltables can be applied to the data by using calmode 'apply'.

The above three calibration modes, 'ps', 'otf', and 'otfraster',
generate sky calibration tables. The user should choose appropriate 
calibration mode depending on the data. Use case for each mode is 
as follows:

    'ps': position switch (including OTF) with explicit
          reference (OFF) spectra
    'otf': non-raster OTF scan without explicit OFFs
           (e.g. Lissajous, double circle, etc.)
           intends to calibrate fast scan data
    'otfraster': raster OTF scan without explicit OFFs

So, if the data contains explicit reference spectra, 'ps' should
be used. Otherwise, 'otfraster' and 'otf' are appropriate for raster 
OTF and non-raster OTF, respectively. In 'otf' and 'otfraster' modes, 
the task first try to find several integrations near edge as OFF 
spectra, then the data are calibrated using those OFFs. If the 
observing pattern is raster, you should use the 'otfraster' mode to 
calibrate data. Otherwise, the 'otf' mode should be used. For detail 
about edge marking, see inline help of sd.edgemarker module and its 
methods. Those modes are designed for OTF observations without 
explicit OFF spectra. However, these modes should work even if 
explicit reference spectra exist. In this case, these spectra will 
be ignored and spectra near edges detected by edge marker will be 
used as reference.

Except for how to choose OFFs, the procedure to derive calibrated
spectra is common for the above three modes. Selected (or preset) 
OFF integrations are separated by its continuity in time domain, 
averaged in each segment, then interpolated to timestamps for ON 
integrations. Effectively, it means that OFF integrations are 
averaged by each OFF spectrum for 'ps' mode, averaged by either ends 
of each raster row for 'otfraster' mode, averaged by each temporal 
segments of detected edges for 'otf' mode. The formula for calibrated 
spectrum is

    Tsys * (ON - OFF) / OFF. 

  
	Arguments :
		infile:	name of input SD dataset (must be in scantable format)
		   Default Value: 

		calmode:	SD calibration mode
		   Default Value: ps
		   Allowed Values:
				ps
				otf
				otfraster
				tsys
				apply
				ps,apply
				ps,tsys,apply
				otf,apply
				otf,tsys,apply
				otfraster,apply
				otfraster,tsys,apply

		fraction:	fraction of the OFF data to mark
		   Default Value: 10%

		noff:	number of the OFF data to mark
		   Default Value: -1

		width:	width of the pixel for edge detection
		   Default Value: 0.5

		elongated:	whether observed area is elongated in one direction or not
		   Default Value: False

		tsysavg:	Whether Tsys is averaged in spectral axis or not
		   Default Value: False

		tsysspw:	list of IF IDs (spectral windows) and their channel ranges of averaging for Tsys calibration.
		   Default Value: 

		applytable:	(List of) sky and/or tsys tables
		   Default Value: 

		interp:	Interpolation type in time[,freq]. Valid options are "nearest", "linear", "cspline", or any numeric string that indicates an order of polynomial interpolation. You can specify interpolation type for time and frequency separately by joining two of the above options by comma (e.g., "linear,cspline").
		   Default Value: 

		spwmap:	A dictionary indicating IFNO combinations to apply Tsys calibration to target. The key should be IFNO for Tsys calibration and its associated value must be a list of science IFNOs to be applied.
		   Default Value: {}

		field:	select data by field IDs and names, e.g. \'3C2*\' (\'\' = all)
		   Default Value: 

		spw:	select data by IF IDs (spectral windows), e.g., \'3,5,7\' (\'\' = all)
		   Default Value: 

		scan:	select data by scan numbers, e.g. \'21~23\' (\'\'=all)
		   Default Value: 

		pol:	select data by polarization IDs, e.g, \'0,1\' (\'\' = all)
		   Default Value: 

		outfile:	name of output file (See a WARNING in help)
		   Default Value: 

		overwrite:	overwrite the output file if already exists
		   Default Value: False

	Returns: void

	Example :

Keyword arguments:
infile -- Name of input SD dataset
calmode -- Calibration mode. If you want to generate calibration table 
           or apply existing calibration tables, set calmode to simple 
           string. On the other hand, if you want to calibrate data 
           on-the-fly, you have to set calmode to a composite calmode 
           string separated by comma. So far, sky calibration has three 
           types, 'ps', 'otf', and 'otfraster'. If observation is 
           configured to observe reference position, calmode must be 
           'ps'. Otherwise, 'otf' or 'otfraster' should be used
           depending on observing pattern. If observing pattern is 
           raster scan, calmode must be 'otfraster' while 'otf' must 
           be used when observing pattern is non-raster 
           (e.g., Lissajous).
        options: 'ps','otf','otfraster','tsys','apply'
        default: 'ps'
        example: Here is an example for composite calmode.
                 'ps,apply' (do sky cal and apply)
                 'ps,tsys,apply' (do sky and Tsys cal and apply)
    >>> calmode expandable parameter
        fraction -- Edge marker parameter of 'otf' and 'otfraster'.
                    Specify a number of OFF integrations (at each
                    side of the raster rows in 'otfraster' mode)
                    as a fraction of total number of integrations.
                    In 'otfraster' mode, number of integrations 
                    to be marked as OFF, n_off, is determined by 
                    the following formula,

                        n_off = floor(fraction * n),

                    where n is number of integrations per raster 
                    row. Note that n_off from both sides will be  
                    marked as OFF so that twice of specified 
                    fraction will be marked at most. For example, 
                    if you specify fraction='10%', resultant 
                    fraction of OFF integrations will be 20% at 
                    most.
                    In 'otf' mode, n_off is given by,

                        n_off = floor(fraction * n),

                    where n is number of total integrations.
                    n_off is used as criterion of iterative marking
                    process. Therefore, resulting total number of 
                    OFFs will be larger than n_off. In practice,
                    fraction is a geometrical fraction of edge
                    region. Thus, if integrations are concentrated
                    on edge region (e.g. some of Lissajous
                    patterns), then resulting n_off may be 
                    unexpectedly large.
                default: '10%'
                options: '20%' in string style or float value less 
                         than 1.0 (e.g. 0.15).
                         'auto' is available only for 'otfraster'. 
        noff -- Edge marking parameter for 'otfraster'.
                It is used to specify a number of OFF spectra near 
                edge directly. Value of noff comes before setting 
                by fraction. Note that n_off from both sides will 
                be marked as OFF so that twice of specified noff 
                will be marked at most.
                default: -1 (use fraction)
                options: any positive integer
        width -- Edge marking parameter for 'otf'.
                 Pixel width with respect to a median spatial 
                 separation between neighboring two data in time.
                 Default will be fine in most cases.
                default: 0.5
                options: float value
        elongated -- Edge marking parameter for 'otf'.
                     Set True only if observed area is elongated 
                     in one direction.
                default: False
        tsysavg -- Whether Tsys is averaged in spectral axis or not.
                default: False
                options: (bool) True, False
        tsysspw -- list of IF IDs (spectral windows) and their channel
                   ranges of averaging for Tsys calibration.
                   It does no effect if you don't want to do Tsys 
                   calibration. the user is able to specify channel
                   range for averaging (effective if tsysavg is True). 
                default: '' (auto-detect tsys spws)
                example: tsysspw='3,5,7' (IF IDs 3,5,7; all channels)
                         tsysspw='<2' (IF IDs less than 2; all channels)
                         tsysspw='1:0~100' (IF ID1; between channels 0 and 100)
        applytable -- List of sky/Tsys calibration tables you want to 
                      apply.
                default: ''
        interp -- Interpolation method in time and frequency axis. 
                  Set comma separated method strings if you want 
                  to use different interpolation in time and 
                  frequency. 
                options: 'linear', 'cspline', 'nearest', 
                         any numeric string indicating an order 
                         of polynomial.
                default: '' (linear in time and frequency)
                example: 'linear,cspline' (linear in time, cubic 
                                           spline in frequency)
                         'linear,3' (linear in time, third order 
                                     polynomial in frequency)
                         'nearest' (nearest in time and frequency)
        spwmap -- Dictionary defining transfer of Tsys calibration. 
                  Key must be IFNO for Tsys and its value must be 
                  a list of IFNOs for science target.
                default: {}
                example: {1: [5,6], 3: [7,8]}
                         Tsys in IFNO1 is transferred to IFNO5, 6 
                         while Tsys in IFNO3 is to IFNO7, 8.
field -- select data by field IDs and names
        default: '' (use all fields)
        example: field='3C2*' (all names starting with 3C2)
                 field='0,4,5~7' (field IDs 0,4,5,6,7)
                 field='0,3C273' (field ID 0 or field named 3C273)
        this selection is in addition to the other selections to data
spw -- select data by IF IDs (spectral windows)
        NOTE this task only supports IF ID selction and ignores channel
        selection.
        default: '' (use all IFs and channels)
        example: spw='3,5,7' (IF IDs 3,5,7; all channels)
                 spw='<2' (IF IDs less than 2, i.e., 0,1; all channels)
                 spw='30~45GHz' (IF IDs with the center frequencies in range 30-45GHz; all channels)
        this selection is in addition to the other selections to data
scan -- select data by scan numbers
        default: '' (use all scans)
        example: scan='21~23' (scan IDs 21,22,23)
        this selection is in addition to the other selections to data
pol -- select data by polarization IDs
        default: '' (use all polarizations)
        example: pol='0,1' (polarization IDs 0,1)
        this selection is in addition to the other selections to data
outfile -- Name of output file
        NOTE if you omit, behavior of the task depends on calmode. 
        If calmode includes 'apply', then omitting outfile indicates 
        that infile is overwritten by the calibrated data. In this case, 
        you have to set overwrite to True. If calmode doesn't include 
        'apply', omitting outfile indicates that the task will use 
        default outfile name based on infile and predefined suffix 
        ('_sky' for sky, '_tsys' for Tsys).
        default: '' (<infile>_<suffix> for calibration 
                     while overwrite infile for apply mode)
overwrite -- overwrite the output file if already exists
        options: (bool) True,False
        default: False
        NOTE this parameter is ignored when outform='ASCII'


DESCRIPTION:

Task sdcal2 is an implementation of a calibration scheme like as 
interferometry, i.e., generate caltables and apply them. Available 
calibration modes are 'ps', 'otf', 'otfraster', and 'tsys'. Those 
modes generates caltables for sky or Tsys calibration. Those 
caltables can be applied to the data by using calmode 'apply'.

The above three calibration modes, 'ps', 'otf', and 'otfraster',
generate sky calibration tables. The user should choose
appropriate calibration mode depending on the data. Use case
for each mode is as follows:

    'ps': position switch (including OTF) with explicit
          reference (OFF) spectra
    'otf': non-raster OTF scan without explicit OFFs
           (e.g. Lissajous, double circle, etc.)
           intends to calibrate fast scan data
    'otfraster': raster OTF scan without explicit OFFs

So, if the data contains explicit reference spectra, 'ps' should
be used. Otherwise, 'otfraster' and 'otf' are appropriate for
raster OTF and non-raster OTF, respectively. In 'otf' and
'otfraster' modes, the task first try to find several integrations 
near edge as OFF spectra, then the data are calibrated using those 
OFFs. If the observing pattern is raster, you 
should use the 'otfraster' mode to calibrate data. Otherwise, the 
'otf' mode should be used. For detail about edge marking, see 
inline help of sd.edgemarker module and its methods.
Those modes are designed for OTF observations without 
explicit OFF spectra. However, these modes should work even if
explicit reference spectra exist. In this case, these spectra
will be ignored and spectra near edges detected by edge
marker will be used as reference.

Except for how to choose OFFs, the procedure to derive calibrated
spectra is common for the
above three modes. Selected (or preset) OFF integrations are
separated by its continuity in time domain, averaged in
each segment, then interpolated to timestamps for ON integrations.
Effectively, it means that OFF integrations are averaged by each
OFF spectrum for 'ps' mode, averaged by either ends of each raster
row for 'otfraster' mode, averaged by each temporal segments of
detected edges for 'otf' mode. The formula for calibrated spectrum
is

    Tsys * (ON - OFF) / OFF. 
  
You can calibrate data on-the-fly like sdcal task by setting 
calmode to a composite calmode string separated by comma. 
For example, calmode='ps,apply' means doing sky calibration and 
apply it on-the-fly. In this case, caltable is generated as a 
temporary plain table and will be deleted at the end.
Allowed calibration modes in this task is as follows:

    ps
        generate sky caltable using 'ps' mode
    otf
        generate sky caltable using 'otf' mode
    otfraster
        generate sky caltable using 'otfraster' mode
    tsys
        generate tsys caltable
    apply
        apply caltables specified by applytable parameter
    ps,apply
        generate temporary sky caltable using 'ps' mode and
        apply it. also apply caltables specified by applytable 
    ps,tsys,apply
        generate temporary sky caltable using 'ps' mode as well
        as temporary tsys caltable, and apply them. 
    otf,apply
        generate temporary sky caltable using 'otf' mode and
        apply it. also apply caltables specified by applytable 
    otf,tsys,apply
        generate temporary sky caltable using 'otf' mode as well
        as temporary tsys caltable, and apply them. 
    otfraster,apply
        generate temporary sky caltable using 'otfraster' mode
        and apply it. also apply caltables specified by applytable 
    otfraster,tsys,apply
        generate temporary sky caltable using 'otfraster' mode
        as well as temporary tsys caltable, and apply them. 

There are several control parameters for sky/Tsys calibration and 
application of caltables. See the above parameter description.

In ALMA, Tsys measurement is usually done using different spectral
setup from spectral windows for science target. In this case, sdcal2
transfers Tsys values to science spectral windows in the application
stage. To do that, the user has to give a list of spectral windows for
Tsys measurement as well as mapping between spectral windows for Tsys
measurement and scicence target. These can be specified by parameters
'tsysspw' and 'spwmap', which are defined as subparameters of 'calmode'.
For example, suppose that Tsys measurements for science windows 17, 19,
21, and 23 are done in spw 9, 11, 13, and 15, respectively. 
In this case, tsysspw and spwmap should be specified as follows:

    tsysspw = '9,11,13,15'
    spwmap = {9:[17],11:[19],13:[21],15:[23]}

Below is an example of full specification of task parameters for calmode
of 'ps,tsys,apply':

    default(sdcal2)
    infile = 'foo.asap'
    calmode = 'ps,tsys,apply'
    spw = ''
    tsysspw = '9,11,13,15'
    spwmap = {9:[17],11:[19],13:[21],15:[23]}
    outfile = 'bar.asap'
    sdcal2()

Note that, in contrast to applycal task, spwmap must be a dictionary
with Tsys spectral window as key and a list of corresponding science
spectral window as value. Note also that the parameter 'spw' should
not be used to specify a list of spectral windows for Tsys measurement.
It is intended to select data to be calibrated so that the list should
contain spectral windows for both science target and Tsys measurement.
The task will fail if you use 'spw' instead of 'tsysspw'. 

For Tsys calibration, the user is able to choose whether Tsys is
averaged in spectral axis or not. If tsysavg is False (default),
resulting Tsys is spectral value. On the other hand, when tsysavg
is True, Tsys is averaged in spectral axis before output. The channel
range for averaging is whole channels by default. If channel range is
specified by tsysspw string, it is used for averaging. The user can
specify channel range with ms selection syntax. For example,

    tsysspw = '1:0~100'

specifies spw 1 for Tsys calibration and channel range between channel
0 and 100 for averaging. You can specify more than one ranges per spw.

    tsysspw = '1:0~100;200~400'

In this case, selected ranges are between 0 and 100 plus 200 and 400.
Note that even if multiple ranges are selected, the task average whole
ranges together and output single averaged value. You can specify multiple
spws by separating comma.

    tsysspw = '1:0~100,3:400~500'

Note that specified channel range is ignored if tsysavg is False.
  
        """
	if not hasattr(self, "__globals__") or self.__globals__ == None :
           self.__globals__=sys._getframe(len(inspect.stack())-1).f_globals
	#casac = self.__globals__['casac']
	casalog = self.__globals__['casalog']
	#casalog = casac.casac.logsink()
        self.__globals__['__last_task'] = 'sdcal2'
        self.__globals__['taskname'] = 'sdcal2'
        ###
        self.__globals__['update_params'](func=self.__globals__['taskname'],printtext=False,ipython_globals=self.__globals__)
        ###
        ###
        #Handle globals or user over-ride of arguments
        #
        if type(self.__call__.func_defaults) is NoneType:
            function_signature_defaults={}
	else:
	    function_signature_defaults=dict(zip(self.__call__.func_code.co_varnames[1:],self.__call__.func_defaults))
	useLocalDefaults = False

        for item in function_signature_defaults.iteritems():
                key,val = item
                keyVal = eval(key)
                if (keyVal == None):
                        #user hasn't set it - use global/default
                        pass
                else:
                        #user has set it - use over-ride
			if (key != 'self') :
			   useLocalDefaults = True

	myparams = {}
	if useLocalDefaults :
	   for item in function_signature_defaults.iteritems():
	       key,val = item
	       keyVal = eval(key)
	       exec('myparams[key] = keyVal')
	       self.parameters[key] = keyVal
	       if (keyVal == None):
	           exec('myparams[key] = '+ key + ' = self.itsdefault(key)')
		   keyVal = eval(key)
		   if(type(keyVal) == dict) :
                      if len(keyVal) > 0 :
		         exec('myparams[key] = ' + key + ' = keyVal[len(keyVal)-1][\'value\']')
		      else :
		         exec('myparams[key] = ' + key + ' = {}')
	 
        else :
            print ''

            myparams['infile'] = infile = self.parameters['infile']
            myparams['calmode'] = calmode = self.parameters['calmode']
            myparams['fraction'] = fraction = self.parameters['fraction']
            myparams['noff'] = noff = self.parameters['noff']
            myparams['width'] = width = self.parameters['width']
            myparams['elongated'] = elongated = self.parameters['elongated']
            myparams['tsysavg'] = tsysavg = self.parameters['tsysavg']
            myparams['tsysspw'] = tsysspw = self.parameters['tsysspw']
            myparams['applytable'] = applytable = self.parameters['applytable']
            myparams['interp'] = interp = self.parameters['interp']
            myparams['spwmap'] = spwmap = self.parameters['spwmap']
            myparams['field'] = field = self.parameters['field']
            myparams['spw'] = spw = self.parameters['spw']
            myparams['scan'] = scan = self.parameters['scan']
            myparams['pol'] = pol = self.parameters['pol']
            myparams['outfile'] = outfile = self.parameters['outfile']
            myparams['overwrite'] = overwrite = self.parameters['overwrite']


	result = None

#
#    The following is work around to avoid a bug with current python translation
#
        mytmp = {}

        mytmp['infile'] = infile
        mytmp['calmode'] = calmode
        mytmp['fraction'] = fraction
        mytmp['noff'] = noff
        mytmp['width'] = width
        mytmp['elongated'] = elongated
        mytmp['tsysavg'] = tsysavg
        mytmp['tsysspw'] = tsysspw
        mytmp['applytable'] = applytable
        mytmp['interp'] = interp
        mytmp['spwmap'] = spwmap
        mytmp['field'] = field
        mytmp['spw'] = spw
        mytmp['scan'] = scan
        mytmp['pol'] = pol
        mytmp['outfile'] = outfile
        mytmp['overwrite'] = overwrite
	pathname='file:///'+os.environ.get('CASAPATH').split()[0]+'/xml/'
        trec = casac.casac.utils().torecord(pathname+'sdcal2.xml')

        casalog.origin('sdcal2')
	try :
          #if not trec.has_key('sdcal2') or not casac.casac.utils().verify(mytmp, trec['sdcal2']) :
	    #return False

          casac.casac.utils().verify(mytmp, trec['sdcal2'], True)
          scriptstr=['']
          saveinputs = self.__globals__['saveinputs']
          if type(self.__call__.func_defaults) is NoneType:
              saveinputs=''
          else:
              saveinputs('sdcal2', 'sdcal2.last', myparams, self.__globals__,scriptstr=scriptstr)
          tname = 'sdcal2'
          spaces = ' '*(18-len(tname))
          casalog.post('\n##########################################'+
                       '\n##### Begin Task: ' + tname + spaces + ' #####')
          if type(self.__call__.func_defaults) is NoneType:
              casalog.post(scriptstr[0]+'\n', 'INFO')
          else :
              casalog.post(scriptstr[1][1:]+'\n', 'INFO')
          result = sdcal2(infile, calmode, fraction, noff, width, elongated, tsysavg, tsysspw, applytable, interp, spwmap, field, spw, scan, pol, outfile, overwrite)
          casalog.post('##### End Task: ' + tname + '  ' + spaces + ' #####'+
                       '\n##########################################')

	except Exception, instance:
          if(self.__globals__.has_key('__rethrow_casa_exceptions') and self.__globals__['__rethrow_casa_exceptions']) :
             raise
          else :
             #print '**** Error **** ',instance
	     tname = 'sdcal2'
             casalog.post('An error occurred running task '+tname+'.', 'ERROR')
             pass

        gc.collect()
        return result
#
#
#
    def paramgui(self, useGlobals=True, ipython_globals=None):
        """
        Opens a parameter GUI for this task.  If useGlobals is true, then any relevant global parameter settings are used.
        """
        import paramgui
	if not hasattr(self, "__globals__") or self.__globals__ == None :
           self.__globals__=sys._getframe(len(inspect.stack())-1).f_globals

        if useGlobals:
	    if ipython_globals == None:
                myf=self.__globals__
            else:
                myf=ipython_globals

            paramgui.setGlobals(myf)
        else:
            paramgui.setGlobals({})

        paramgui.runTask('sdcal2', myf['_ip'])
        paramgui.setGlobals({})

#
#
#
    def defaults(self, param=None, ipython_globals=None, paramvalue=None, subparam=None):
	if not hasattr(self, "__globals__") or self.__globals__ == None :
           self.__globals__=sys._getframe(len(inspect.stack())-1).f_globals
        if ipython_globals == None:
            myf=self.__globals__
        else:
            myf=ipython_globals

        a = odict()
        a['infile']  = ''
        a['calmode']  = 'ps'
        a['field']  = ''
        a['spw']  = ''
        a['scan']  = ''
        a['pol']  = ''
        a['outfile']  = ''
        a['overwrite']  = False

        a['calmode'] = {
                    0:{'value':'ps'}, 
                    1:odict([{'value':'otf'}, {'fraction':'10%'}, {'width':0.5}, {'elongated':False}]), 
                    2:odict([{'value':'otfraster'}, {'fraction':'10%'}, {'noff':-1}]), 
                    3:odict([{'value':'tsys'}, {'tsysavg':False}, {'tsysspw':''}]), 
                    4:odict([{'value':'apply'}, {'applytable':''}, {'interp':''}, {'spwmap':{}}]), 
                    5:odict([{'value':'ps,apply'}, {'applytable':''}, {'interp':''}, {'spwmap':{}}]), 
                    6:odict([{'value':'ps,tsys,apply'}, {'tsysavg':False}, {'tsysspw':''}, {'applytable':''}, {'interp':''}, {'spwmap':{}}]), 
                    7:odict([{'value':'otf,apply'}, {'fraction':'10%'}, {'width':0.5}, {'elongated':False}, {'applytable':''}, {'interp':''}, {'spwmap':{}}]), 
                    8:odict([{'value':'otf,tsys,apply'}, {'fraction':'10%'}, {'width':0.5}, {'elongated':False}, {'tsysavg':False}, {'tsysspw':''}, {'applytable':''}, {'interp':''}, {'spwmap':{}}]), 
                    9:odict([{'value':'otfraster,apply'}, {'fraction':'10%'}, {'noff':-1}, {'applytable':''}, {'interp':''}, {'spwmap':{}}]), 
                    10:odict([{'value':'otfraster,tsys,apply'}, {'fraction':'10%'}, {'noff':-1}, {'tsysavg':False}, {'tsysspw':''}, {'applytable':''}, {'interp':''}, {'spwmap':{}}])}

### This function sets the default values but also will return the list of
### parameters or the default value of a given parameter
        if(param == None):
                myf['__set_default_parameters'](a)
        elif(param == 'paramkeys'):
                return a.keys()
        else:
            if(paramvalue==None and subparam==None):
               if(a.has_key(param)):
                  return a[param]
               else:
                  return self.itsdefault(param)
            else:
               retval=a[param]
               if(type(a[param])==dict):
                  for k in range(len(a[param])):
                     valornotval='value'
                     if(a[param][k].has_key('notvalue')):
                        valornotval='notvalue'
                     if((a[param][k][valornotval])==paramvalue):
                        retval=a[param][k].copy()
                        retval.pop(valornotval)
                        if(subparam != None):
                           if(retval.has_key(subparam)):
                              retval=retval[subparam]
                           else:
                              retval=self.itsdefault(subparam)
		     else:
                        retval=self.itsdefault(subparam)
               return retval


#
#
    def check_params(self, param=None, value=None, ipython_globals=None):
      if ipython_globals == None:
          myf=self.__globals__
      else:
          myf=ipython_globals
#      print 'param:', param, 'value:', value
      try :
         if str(type(value)) != "<type 'instance'>" :
            value0 = value
            value = myf['cu'].expandparam(param, value)
            matchtype = False
            if(type(value) == numpy.ndarray):
               if(type(value) == type(value0)):
                  myf[param] = value.tolist()
               else:
                  #print 'value:', value, 'value0:', value0
                  #print 'type(value):', type(value), 'type(value0):', type(value0)
                  myf[param] = value0
                  if type(value0) != list :
                     matchtype = True
            else :
               myf[param] = value
            value = myf['cu'].verifyparam({param:value})
            if matchtype:
               value = False
      except Exception, instance:
         #ignore the exception and just return it unchecked
         myf[param] = value
      return value
#
#
    def description(self, key='sdcal2', subkey=None):
        desc={'sdcal2': 'ASAP SD calibration task',
               'infile': 'name of input SD dataset (must be in scantable format)',
               'calmode': 'SD calibration mode [\'ps\',\'otf\',\'otfraster\',\'tsys\',\'apply\', and their possible combinations]',
               'fraction': 'fraction of the OFF data to mark',
               'noff': 'number of the OFF data to mark',
               'width': 'width of the pixel for edge detection',
               'elongated': 'whether observed area is elongated in one direction or not',
               'tsysavg': 'Whether Tsys is averaged in spectral axis or not',
               'tsysspw': 'list of IF IDs (spectral windows) and their channel ranges of averaging for Tsys calibration.',
               'applytable': '(List of) sky and/or tsys tables',
               'interp': 'Interpolation type in time[,freq]. Valid options are "nearest", "linear", "cspline", or any numeric string that indicates an order of polynomial interpolation. You can specify interpolation type for time and frequency separately by joining two of the above options by comma (e.g., "linear,cspline").',
               'spwmap': 'A dictionary indicating IFNO combinations to apply Tsys calibration to target. The key should be IFNO for Tsys calibration and its associated value must be a list of science IFNOs to be applied.',
               'field': 'select data by field IDs and names, e.g. \'3C2*\' (\'\' = all)',
               'spw': 'select data by IF IDs (spectral windows), e.g., \'3,5,7\' (\'\' = all)',
               'scan': 'select data by scan numbers, e.g. \'21~23\' (\'\'=all)',
               'pol': 'select data by polarization IDs, e.g, \'0,1\' (\'\' = all)',
               'outfile': 'name of output file (See a WARNING in help)',
               'overwrite': 'overwrite the output file if already exists [True, False]',

              }

#
# Set subfields defaults if needed
#

        if(desc.has_key(key)) :
           return desc[key]

    def itsdefault(self, paramname) :
        a = {}
        a['infile']  = ''
        a['calmode']  = 'ps'
        a['fraction']  = '10%'
        a['noff']  = -1
        a['width']  = 0.5
        a['elongated']  = False
        a['tsysavg']  = False
        a['tsysspw']  = ''
        a['applytable']  = ''
        a['interp']  = ''
        a['spwmap']  = {}
        a['field']  = ''
        a['spw']  = ''
        a['scan']  = ''
        a['pol']  = ''
        a['outfile']  = ''
        a['overwrite']  = False

        #a = sys._getframe(len(inspect.stack())-1).f_globals

        if self.parameters['calmode']  == 'otf':
            a['fraction'] = '10%'
            a['width'] = 0.5
            a['elongated'] = False

        if self.parameters['calmode']  == 'otfraster':
            a['fraction'] = '10%'
            a['noff'] = -1

        if self.parameters['calmode']  == 'tsys':
            a['tsysavg'] = False
            a['tsysspw'] = ''

        if self.parameters['calmode']  == 'apply':
            a['applytable'] = ''
            a['interp'] = ''
            a['spwmap'] = {}

        if self.parameters['calmode']  == 'ps,apply':
            a['applytable'] = ''
            a['interp'] = ''
            a['spwmap'] = {}

        if self.parameters['calmode']  == 'ps,tsys,apply':
            a['tsysavg'] = False
            a['tsysspw'] = ''
            a['applytable'] = ''
            a['interp'] = ''
            a['spwmap'] = {}

        if self.parameters['calmode']  == 'otf,apply':
            a['fraction'] = '10%'
            a['width'] = 0.5
            a['elongated'] = False
            a['applytable'] = ''
            a['interp'] = ''
            a['spwmap'] = {}

        if self.parameters['calmode']  == 'otf,tsys,apply':
            a['fraction'] = '10%'
            a['width'] = 0.5
            a['elongated'] = False
            a['tsysavg'] = False
            a['tsysspw'] = ''
            a['applytable'] = ''
            a['interp'] = ''
            a['spwmap'] = {}

        if self.parameters['calmode']  == 'otfraster,apply':
            a['fraction'] = '10%'
            a['noff'] = -1
            a['applytable'] = ''
            a['interp'] = ''
            a['spwmap'] = {}

        if self.parameters['calmode']  == 'otfraster,tsys,apply':
            a['fraction'] = '10%'
            a['noff'] = -1
            a['tsysavg'] = False
            a['tsysspw'] = ''
            a['applytable'] = ''
            a['interp'] = ''
            a['spwmap'] = {}

        if a.has_key(paramname) :
	      return a[paramname]
sdcal2_cli = sdcal2_cli_()
